function [ out ] = predprey( varargin )

if nargin
    action    = varargin{1};
else
    action    = 'test';
end %if nargin

switch action
    case 'test'
        
        th          = struct;
        th.a        = 0.08;
        th.b        = 0.002;
        th.c        = 0.0004;
        th.d        = 0.2;
        th.x1cap    = 1000;
        th.x1_0     = 500;
        th.x2_0     = 10;
        
        % --- control ---
        FINAL_TIME  = 250;
        INITIAL_TIME= 0;
        TIME_STEP   = 0.25;
        
        % --- DATA ---
        DATA            = struct;
        DATA.tStep      = 0.25;
        DATA.t0         = INITIAL_TIME;
        DATA.tf         = FINAL_TIME;
        DATA.t          = (DATA.t0:DATA.tStep:DATA.tf)';
        
        DATA.th(1)      = th;
        th.x1cap        = 10^8;
        DATA.th(2)      = th;
        DATA.noRuns     = numel(DATA.th);
        
        for ii = 1:DATA.noRuns
            [~, DATA.y{ii}] = rk4(@f,[DATA.t0 DATA.tf],...
                [DATA.th(ii).x1_0; DATA.th(ii).x2_0],DATA.tStep);
            DATA.eqStat{ii}            = equilibria(DATA.th(ii),'get');
        end %for ii
        
    case 'range'
        
        clear DATA
        
        d               = 0.001;
        noRuns          = 25;
        pars            = 2*pi()*rand(7,noRuns);
        
        % --- control ---
        FINAL_TIME      = 250;
        INITIAL_TIME    = 0;
        TIME_STEP       = 0.25;
        
        % --- DATA ---
        DATA            = struct;
        DATA.tStep      = 0.25;
        DATA.t0         = INITIAL_TIME;
        DATA.tf         = FINAL_TIME;
        DATA.t          = (DATA.t0:DATA.tStep:DATA.tf)';
        
        DATA.noRuns     = noRuns;
        
        for ii = 1:DATA.noRuns
            thTEMP          = struct;
            thTEMP.a        = d  + 0.5*(0.8  -d)*(1+cos(pars(1,ii)));
            thTEMP.b        = d  + 0.5*(0.02 -d)*(1+cos(pars(2,ii)));
            thTEMP.c        = d  + 0.5*(0.004-d)*(1+cos(pars(3,ii)));
            thTEMP.d        = d  + 0.5*(0.5  -d)*(1+cos(pars(4,ii)));
            thTEMP.x1cap    = 100+ 0.5*(900  -d)*(1+cos(pars(5,ii)));
            thTEMP.x1_0     = d  + 0.5*(1500-d).*(1+cos(pars(6,ii)));
            thTEMP.x2_0     = d  + 0.5*(250-d).*(1+cos(pars(7,ii)));
            DATA.th(ii)     = thTEMP;
                       
            [~, DATA.y{ii}] = rk4(@f,[DATA.t0 DATA.tf],...
                [DATA.th(ii).x1_0; DATA.th(ii).x2_0],DATA.tStep);
            DATA.eqStat{ii} = equilibria(DATA.th(ii),'get');
        end %for ii
            
    case 'equilibrium'
        
        % clear DATA
        
        syms a b c d x1 x2 x1cap 
        
        syms fSym(x1, x2)
        fSym(x1, x2)    = [a*x1* (1 - x1/x1cap) - b*x1*x2; c*x1*x2 - d*x2];
        
        xSym    = [x1; x2];
        
        % --- static equilibria ---
        xEq = solve('a*x1* (1 - x1/x1cap) - b*x1*x2 = 0', ...
                    'c*x1*x2 - d*x2 = 0',...
                    x1,x2);
        
        try noEqs = size(xEq.x1);
        catch
            error('No static equilibria found')
        end %try
        
        for ss = 1:noEqs
            DATA.eqStat{ss}     = [xEq.x1(ss); xEq.x2(ss)];
        end %for ss
        
        % --- stability equilibria ---
        J   = jacobian(fSym,xSym)
        
    otherwise
        try
            load varargin{1}
        catch
            1;
        end
end %switch action

% --- draw ---
switch action
    case 'test'
        lnColors    = {'-.' '-'};
        eqnColors   = {'r+' 'ko'};
        
        figure(101),clf
        
        subplot(2,1,2)
        [ax1, lnPrey1, lnPredator1]     = plotyy(DATA.t,DATA.y{1}(:,1),DATA.t,DATA.y{1}(:,2));
        
        set(get(ax1(1),'Ylabel'),'String','Prey Biomass Units','Interpreter','LaTex');
        set(get(ax1(2),'Ylabel'),'String','Predator Biomass Units','Interpreter','LaTex');
        xlabel('Time [weeks]','Interpreter','LaTex')
        title('$x_{1,\mathrm{cap}}=1000$ [prey biomass units]','Interpreter','LaTex')
        
        subplot(2,1,1)
        [ax2, lnPrey2, lnPredator2]     = plotyy(DATA.t,DATA.y{2}(:,1),DATA.t,DATA.y{2}(:,2));
        
        set(get(ax2(1),'Ylabel'),'String','Prey Biomass Units','Interpreter','LaTex');
        set(get(ax2(2),'Ylabel'),'String','Predator Biomass Units','Interpreter','LaTex');
        title('$x_{1,\mathrm{cap}}=10^8\gg x_1$ [prey biomass units]','Interpreter','LaTex')
        
        % print -depsc 'C:\Users\Jurgen\Documents\TU\Thesis\Literature Study\ExplanationVensim\predPreyTh1Cmp'
        
        figure(102),clf
        
        for ff = 1:2
            hold all
            plot(DATA.y{ff}(:,1),DATA.y{ff}(:,2),lnColors{ff});
            for qq = 3
                plot(DATA.eqStat{ff}{qq}(1),DATA.eqStat{ff}{qq}(2),eqnColors{ff});
            end %for qq
        end %for ee
        xlim([200, 1000]);
        ylim([0, 150]);
        
        leg     = legend('$x_{1,\mathrm{cap}}=1000$','stationary point with $x_{1,\mathrm{cap}}=1000$',...
            '$x_{1,\mathrm{cap}}=10^8\gg x_1$','stationary point with $x_{1,\mathrm{cap}}=10^8$');
        set(leg,'Interpreter','LaTex');
        legPos  = get(leg,'Position');
        legDx   = 0.04;
        legPos  = [legPos(1)-legDx legPos(2) legPos(3)+legDx legPos(4)];
        set(leg,'Position',legPos);
        xlabel('Prey Biomass Units','Interpreter','LaTex')
        ylabel('Predator Biomass Units','Interpreter','LaTex')
        % title('State trajectories in the state space ','Interpreter','LaTex')
        
        % print -depsc 'C:\Users\Jurgen\Documents\TU\Thesis\Literature Study\ExplanationVensim\predPreyCycle'
        
    case 'range'
        
        figure(111),clf
        
        subplot(2,1,1)
        for dd = 1:DATA.noRuns
            hold all
            plot(DATA.t, DATA.y{dd}(:,1));
        end %for dd
        subplot(2,1,2)
        for ee = 1:DATA.noRuns
            hold all
            plot(DATA.t, DATA.y{ee}(:,2));   
        end %for ff
        
        subplot(2,1,1)
        ylim([0,500])
        ylabel('Prey Biomass Units','Interpreter','LaTex');
        subplot(2,1,2)
        ylim([0,250])
        ylabel('Predator Biomass Units','Interpreter','LaTex');
        xlabel('Time [weeks]','Interpreter','LaTex');
        
        % print -depsc 'C:\Users\Jurgen\Documents\TU\Thesis\Literature Study\ExplanationVensim\predPreyRange'
        
        figure(112),clf
        plot(DATA.eqStat{1}{3}(1),DATA.eqStat{1}{3}(2),'k+');
        for ff = 1:DATA.noRuns
            hold all
            plot(DATA.y{ff}(:,1),DATA.y{ff}(:,2));
        end %for ee
        for ff = 1:DATA.noRuns
            plot(DATA.eqStat{ff}{3}(1),DATA.eqStat{ff}{3}(2),'k+');
        end
        xlim([0, 500])
        ylim([0, 250])
        
        legend('equilibrium point');
        
        xlabel('Prey Biomass Units','Interpreter','LaTex')
        ylabel('Predator Biomass Units','Interpreter','LaTex')
        
        % print -depsc 'C:\Users\Jurgen\Documents\TU\Thesis\Literature Study\ExplanationVensim\predPreyRangeCycle'
        
    otherwise
        error('.')
end %switch action

if nargout == 0
    assignin('base','DATA',DATA)
else
    out     = DATA;
end

    function [ out ] = f(t,x)
        
        % out     = [];
        
        % dx      = zeros(size(x));
        % dx(1)   = DATA.th(ii).a * x(1) * (1-x(1)/DATA.th(ii).x1cap) - DATA.th(ii).b * x(1) * x(2);
        % dx(2)   = DATA.th(ii).c * x(1) * x(2) - DATA.th(ii).d * x(2);
        
        % out     = dx;
        
        out = [ DATA.th(ii).a * x(1) * (1-x(1)/DATA.th(ii).x1cap) - DATA.th(ii).b * x(1) * x(2);
            DATA.th(ii).c * x(1) * x(2) - DATA.th(ii).d * x(2)];
        
    end %function [ out ] = f(t,x)

    function [ out ] = equilibria(th,action)
        
        switch action
            case 'get'
                
                eqStat{1}   = [        0;                       0                               ];
                eqStat{2}   = [ th.x1cap;                       0                               ];
                eqStat{3}   = [th.d/th.c; -(th.a*th.d - th.a*th.c*th.x1cap)/(th.b*th.c*th.x1cap)];
                
                out         = eqStat;
                
            otherwise
                error('.')
        end %switch action
        
    end %function [ out ] = equilibria(action)

end %function [ out ] = predprey( varargin )

